package com.sun.media.sound;

import java.io.SequenceInputStream;
import java.io.ByteArrayInputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayOutputStream;
import javax.sound.sampled.AudioSystem;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioFileFormat;

public final class AiffFileWriter extends SunFileWriter
{
    private static final int DOUBLE_MANTISSA_LENGTH = 52;
    private static final int DOUBLE_EXPONENT_LENGTH = 11;
    private static final long DOUBLE_SIGN_MASK = Long.MIN_VALUE;
    private static final long DOUBLE_EXPONENT_MASK = 9218868437227405312L;
    private static final long DOUBLE_MANTISSA_MASK = 4503599627370495L;
    private static final int DOUBLE_EXPONENT_OFFSET = 1023;
    private static final int EXTENDED_EXPONENT_OFFSET = 16383;
    private static final int EXTENDED_MANTISSA_LENGTH = 63;
    private static final int EXTENDED_EXPONENT_LENGTH = 15;
    private static final long EXTENDED_INTEGER_MASK = Long.MIN_VALUE;
    
    public AiffFileWriter() {
        super(new AudioFileFormat.Type[] { AudioFileFormat.Type.AIFF });
    }
    
    @Override
    public AudioFileFormat.Type[] getAudioFileTypes(AudioInputStream audioInputStream) {
        AudioFileFormat.Type[] array = new AudioFileFormat.Type[this.types.length];
        System.arraycopy(this.types, 0, array, 0, this.types.length);
        AudioFormat.Encoding encoding = audioInputStream.getFormat().getEncoding();
        if (AudioFormat.Encoding.ALAW.equals(encoding) || AudioFormat.Encoding.ULAW.equals(encoding) || AudioFormat.Encoding.PCM_SIGNED.equals(encoding) || AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding)) {
            return array;
        }
        return new AudioFileFormat.Type[0];
    }
    
    @Override
    public int write(AudioInputStream audioInputStream, AudioFileFormat.Type type, OutputStream outputStream) throws IOException {
        AiffFileFormat aiffFileFormat = (AiffFileFormat)this.getAudioFileFormat(type, audioInputStream);
        if (audioInputStream.getFrameLength() == -1L) {
            throw new IOException("stream length not specified");
        }
        return this.writeAiffFile(audioInputStream, aiffFileFormat, outputStream);
    }
    
    @Override
    public int write(AudioInputStream audioInputStream, AudioFileFormat.Type type, File file) throws IOException {
        AiffFileFormat aiffFileFormat = (AiffFileFormat)this.getAudioFileFormat(type, audioInputStream);
        BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(new FileOutputStream(file), 4096);
        int writeAiffFile = this.writeAiffFile(audioInputStream, aiffFileFormat, bufferedOutputStream);
        bufferedOutputStream.close();
        if (aiffFileFormat.getByteLength() == -1) {
            int n = aiffFileFormat.getFormat().getChannels() * aiffFileFormat.getFormat().getSampleSizeInBits();
            int n2 = writeAiffFile - aiffFileFormat.getHeaderSize() + 16;
            int v = (int)((long)(n2 - 16) * 8L / (long)n);
            RandomAccessFile randomAccessFile = new RandomAccessFile(file, "rw");
            randomAccessFile.skipBytes(4);
            randomAccessFile.writeInt(writeAiffFile - 8);
            randomAccessFile.skipBytes(4 + aiffFileFormat.getFverChunkSize() + 4 + 4 + 2);
            randomAccessFile.writeInt(v);
            randomAccessFile.skipBytes(16);
            randomAccessFile.writeInt(n2 - 8);
            randomAccessFile.close();
        }
        return writeAiffFile;
    }
    
    private AudioFileFormat getAudioFileFormat(AudioFileFormat.Type type, AudioInputStream audioInputStream) {
        AudioFormat.Encoding pcm_SIGNED = AudioFormat.Encoding.PCM_SIGNED;
        AudioFormat format = audioInputStream.getFormat();
        AudioFormat.Encoding encoding = format.getEncoding();
        boolean n = false;
        if (!this.types[0].equals(type)) {
            throw new IllegalArgumentException("File type " + type + " not supported.");
        }
        AudioFormat.Encoding encoding2;
        int sampleSizeInBits;
        if (AudioFormat.Encoding.ALAW.equals(encoding) || AudioFormat.Encoding.ULAW.equals(encoding)) {
            if (format.getSampleSizeInBits() == 8) {
                encoding2 = AudioFormat.Encoding.PCM_SIGNED;
                sampleSizeInBits = 16;
                n = true;
            }
            else {
                throw new IllegalArgumentException("Encoding " + encoding + " supported only for 8-bit data.");
            }
        }
        else if (format.getSampleSizeInBits() == 8) {
            encoding2 = AudioFormat.Encoding.PCM_UNSIGNED;
            sampleSizeInBits = 8;
        }
        else {
            encoding2 = AudioFormat.Encoding.PCM_SIGNED;
            sampleSizeInBits = format.getSampleSizeInBits();
        }
        AudioFormat o = new AudioFormat(encoding2, format.getSampleRate(), sampleSizeInBits, format.getChannels(), format.getFrameSize(), format.getFrameRate(), true);
        int n2;
        if (audioInputStream.getFrameLength() != -1L) {
            if (!(n == 0)) {
                n2 = (int)audioInputStream.getFrameLength() * format.getFrameSize() * 2 + 54;
            }
            else {
                n2 = (int)audioInputStream.getFrameLength() * format.getFrameSize() + 54;
            }
        }
        else {
            n2 = -1;
        }
        return new AiffFileFormat(AudioFileFormat.Type.AIFF, n2, o, (int)audioInputStream.getFrameLength());
    }
    
    private int writeAiffFile(InputStream inputStream, AiffFileFormat aiffFileFormat, OutputStream outputStream) throws IOException {
        int n = 0;
        InputStream fileStream = this.getFileStream(aiffFileFormat, inputStream);
        byte[] array = new byte[4096];
        int byteLength = aiffFileFormat.getByteLength();
        int read;
        while ((read = fileStream.read(array)) >= 0) {
            if (byteLength > 0) {
                if (read < byteLength) {
                    outputStream.write(array, 0, read);
                    n += read;
                    byteLength -= read;
                }
                else {
                    outputStream.write(array, 0, byteLength);
                    n += byteLength;
                    break;
                }
            }
            else {
                outputStream.write(array, 0, read);
                n += read;
            }
        }
        return n;
    }
    
    private InputStream getFileStream(AiffFileFormat aiffFileFormat, InputStream inputStream) throws IOException {
        AudioFormat format = aiffFileFormat.getFormat();
        int headerSize = aiffFileFormat.getHeaderSize();
        aiffFileFormat.getFverChunkSize();
        int commChunkSize = aiffFileFormat.getCommChunkSize();
        int n = -1;
        int n2 = -1;
        aiffFileFormat.getSsndChunkOffset();
        int v = (short)format.getChannels();
        int v2 = (short)format.getSampleSizeInBits();
        int n3 = v * v2;
        int frameLength = aiffFileFormat.getFrameLength();
        if (frameLength != -1) {
            long n4 = (long)frameLength * (long)n3 / 8L;
            n2 = (int)n4 + 16;
            n = (int)n4 + headerSize;
        }
        float sampleRate = format.getSampleRate();
        if ((inputStream instanceof AudioInputStream)) {
            AudioFormat format2 = ((AudioInputStream)inputStream).getFormat();
            AudioFormat.Encoding encoding = format2.getEncoding();
            if (AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding) || (AudioFormat.Encoding.PCM_SIGNED.equals(encoding) && !format2.isBigEndian())) {
                inputStream = AudioSystem.getAudioInputStream(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, format2.getSampleRate(), format2.getSampleSizeInBits(), format2.getChannels(), format2.getFrameSize(), format2.getFrameRate(), true), (AudioInputStream)inputStream);
            }
            else if (AudioFormat.Encoding.ULAW.equals(encoding) || AudioFormat.Encoding.ALAW.equals(encoding)) {
                if (format2.getSampleSizeInBits() != 8) {
                    throw new IllegalArgumentException("unsupported encoding");
                }
                inputStream = AudioSystem.getAudioInputStream(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, format2.getSampleRate(), format2.getSampleSizeInBits() * 2, format2.getChannels(), format2.getFrameSize() * 2, format2.getFrameRate(), true), (AudioInputStream)inputStream);
            }
        }
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DataOutputStream o = new DataOutputStream(out);
        o.writeInt(1179603533);
        o.writeInt(n - 8);
        o.writeInt(1095321158);
        o.writeInt(1129270605);
        o.writeInt(commChunkSize - 8);
        o.writeShort(v);
        o.writeInt(frameLength);
        o.writeShort(v2);
        this.write_ieee_extended(o, sampleRate);
        o.writeInt(1397968452);
        o.writeInt(n2 - 8);
        o.writeInt(0);
        o.writeInt(0);
        o.close();
        return new SequenceInputStream(new ByteArrayInputStream(out.toByteArray()), new SunFileWriter.NoCloseInputStream(inputStream));
    }
    
    private void write_ieee_extended(DataOutputStream dataOutputStream, float n) throws IOException {
        long doubleToLongBits = Double.doubleToLongBits((double)n);
        long n2 = (doubleToLongBits & Long.MIN_VALUE) >> 63;
        long n3 = (doubleToLongBits & 0x7FF0000000000000L) >> 52;
        long n4 = doubleToLongBits & 0xFFFFFFFFFFFFFL;
        long n5 = n3 - 1023L + 16383L;
        long n6 = n4 << 11;
        int v = (short)(int)(n2 << 15 | n5);
        long v2 = Long.MIN_VALUE | n6;
        dataOutputStream.writeShort(v);
        dataOutputStream.writeLong(v2);
    }
}
